Struct moonshine_util::expect::Expect
source · pub struct Expect<T: QueryData>(/* private fields */);Expand description
A QueryData decorator which panics if its inner query does not match.
Usage
This decorator is useful for preventing systems from silently skipping over entities which may erroneously not a query.
Consider the following erroneous example:
use bevy::prelude::*;
#[derive(Component)]
struct A;
#[derive(Component)]
struct B;
// A and B are always expected to be inserted together:
#[derive(Bundle)]
struct AB {
a: A,
b: B,
}
fn bad_system(mut commands: Commands) {
commands.spawn(A); // Spawn A without B!
}
fn unsafe_system(q: Query<(&A, &B)>) {
for _ in q.iter() {
// An instance of `A` does exist.
// But because `A` does not exist *with* `B`, this system skips over it silently.
}
}This problem can be solved with Expect:
use moonshine_util::expect::Expect;
fn safe_system(q: Query<(&A, Expect<&B>)>) {
for _ in q.iter() {
// This system will panic if it finds an instance of `A` without `B`.
}
}Trait Implementations
sourceimpl<T: QueryData> WorldQuery for Expect<T>
impl<T: QueryData> WorldQuery for Expect<T>
type Fetch<'w> = ExpectFetch<'w, T>
type Fetch<'w> = ExpectFetch<'w, T>
Per archetype/table state used by this
WorldQuery to fetch Self::Itemtype Item<'w> = <T as WorldQuery>::Item<'w>
type Item<'w> = <T as WorldQuery>::Item<'w>
The item returned by this
WorldQuery
For QueryData this will be the item returned by the query.
For QueryFilter this will be either (), or a bool indicating whether the entity should be included
or a tuple of such things.type State = <T as WorldQuery>::State
type State = <T as WorldQuery>::State
State used to construct a
Self::Fetch. This will be cached inside QueryState,
so it is best to move as much data / computation here as possible to reduce the cost of
constructing Self::Fetch.sourcefn shrink<'wlong: 'wshort, 'wshort>(
item: Self::Item<'wlong>,
) -> Self::Item<'wshort>
fn shrink<'wlong: 'wshort, 'wshort>( item: Self::Item<'wlong>, ) -> Self::Item<'wshort>
This function manually implements subtyping for the query items.
sourceconst IS_DENSE: bool = T::IS_DENSE
const IS_DENSE: bool = T::IS_DENSE
Returns true if (and only if) every table of every archetype matched by this fetch contains
all of the matched components. This is used to select a more efficient “table iterator”
for “dense” queries. If this returns true,
WorldQuery::set_table must be used before
WorldQuery::fetch can be called for iterators. If this returns false,
WorldQuery::set_archetype must be used before WorldQuery::fetch can be called for
iterators.sourceunsafe fn init_fetch<'w>(
world: UnsafeWorldCell<'w>,
state: &T::State,
last_run: Tick,
this_run: Tick,
) -> ExpectFetch<'w, T>
unsafe fn init_fetch<'w>( world: UnsafeWorldCell<'w>, state: &T::State, last_run: Tick, this_run: Tick, ) -> ExpectFetch<'w, T>
Creates a new instance of this fetch. Read more
sourceunsafe fn set_archetype<'w>(
fetch: &mut ExpectFetch<'w, T>,
state: &T::State,
archetype: &'w Archetype,
table: &'w Table,
)
unsafe fn set_archetype<'w>( fetch: &mut ExpectFetch<'w, T>, state: &T::State, archetype: &'w Archetype, table: &'w Table, )
Adjusts internal state to account for the next
Archetype. This will always be called on
archetypes that match this WorldQuery. Read moresourceunsafe fn set_table<'w>(
fetch: &mut ExpectFetch<'w, T>,
state: &T::State,
table: &'w Table,
)
unsafe fn set_table<'w>( fetch: &mut ExpectFetch<'w, T>, state: &T::State, table: &'w Table, )
Adjusts internal state to account for the next
Table. This will always be called on tables
that match this WorldQuery. Read moresourceunsafe fn fetch<'w>(
fetch: &mut Self::Fetch<'w>,
entity: Entity,
table_row: TableRow,
) -> Self::Item<'w>
unsafe fn fetch<'w>( fetch: &mut Self::Fetch<'w>, entity: Entity, table_row: TableRow, ) -> Self::Item<'w>
Fetch
Self::Item for either the given entity in the current Table,
or for the given entity in the current Archetype. This must always be called after
WorldQuery::set_table with a table_row in the range of the current Table or after
WorldQuery::set_archetype with a entity in the current archetype. Read moresourcefn update_component_access(
state: &T::State,
access: &mut FilteredAccess<ComponentId>,
)
fn update_component_access( state: &T::State, access: &mut FilteredAccess<ComponentId>, )
sourcefn init_state(world: &mut World) -> T::State
fn init_state(world: &mut World) -> T::State
Creates and initializes a
State for this WorldQuery type.sourcefn matches_component_set(
_state: &T::State,
_set_contains_id: &impl Fn(ComponentId) -> bool,
) -> bool
fn matches_component_set( _state: &T::State, _set_contains_id: &impl Fn(ComponentId) -> bool, ) -> bool
sourcefn set_access(_state: &mut Self::State, _access: &FilteredAccess<ComponentId>)
fn set_access(_state: &mut Self::State, _access: &FilteredAccess<ComponentId>)
Sets available accesses for implementors with dynamic access such as
FilteredEntityRef
or FilteredEntityMut. Read moreimpl<T: ReadOnlyQueryData> ReadOnlyQueryData for Expect<T>